@@ -1,4 +1,4 @@
-/*$Id: c_insert.cc,v 14.12 1999/07/23 10:13:54 al Exp $ -*- C++ -*-
+/*$Id: c_insert.cc,v 17.1 2000/02/19 08:51:13 al Exp $ -*- C++ -*-
  * insert command
  * insert new nodes
  * Doesn't really work.
@@ -34,9 +34,9 @@ void CMD::insert(CS& cmd)
 	 ci=CARD_LIST::card_list.begin();ci!=CARD_LIST::card_list.end();++ci){
     CARD* brh = *ci;
     if (brh->is_device()){
-      for (int ii = 0;  brh->n[ii].e != INVALIDNODE;  ii++){
-	if (brh->n[ii].e >= nod)		/* move them up */
-	  brh->n[ii].e += ncnt;
+      for (int ii = 0;  brh->_n[ii].e != INVALIDNODE;  ii++){
+	if (brh->_n[ii].e >= nod)		/* move them up */
+	  brh->_n[ii].e += ncnt;
       }
     }
   untested();
