@@ -1,4 +1,4 @@
-/*$Id: d_cccs.h,v 15.11 1999/10/14 07:46:49 al Exp $ -*- C++ -*-
+/*$Id: d_cccs.h,v 17.21 2000/05/02 09:32:08 al Exp $ -*- C++ -*-
  * current controlled current source
  */
 #ifndef D_CCCS_H
@@ -11,7 +11,9 @@ private:
 public:
   explicit DEV_CCCS()		:CCSRC_BASE() {}
 private: // override virtual
+  char	   id_letter()const	{return 'F';}
   const char* dev_type()const	{return "cccs";}
+  //int	   numnodes()const	//CCSRC_BASE=4
   CARD*	   clone()const		{untested(); return new DEV_CCCS(*this);}
   //void   parse(CS&);		//ELEMENT
   //void   print(OMSTREAM,int)const; //CCSRC_BASE
@@ -30,11 +32,16 @@ private: // override virtual
   //double tr_review();		//CARD/nothing
   //void   tr_accept();		//CARD/nothing
   //void   tr_unload();		//CCSRC_BASE
+  //double tr_amps()const	//ELEMENT
   //double tr_involts()const;	//CCSRC_BASE
+  //double tr_involts_limited()const;	//CCSRC_BASE
+  //double tr_probe_num(CS&)const;//ELEMENT
 
-  void	    ac_begin()		{ev = y0.f1;}
+  void	    ac_begin()		{_ev = _y0.f1;}
   void	    do_ac();
+  void	    ac_load()		{unreachable(); incomplete(); /*fake*/}
   //COMPLEX ac_involts()const;	//CCSRC_BASE
+  //XPROBE  ac_probe_ext(CS&)const;//ELEMENT
 };
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
