@@ -1,4 +1,4 @@
-/*$Id: d_coment.cc,v 14.19 1999/08/18 17:17:11 al Exp $ -*- C++ -*-
+/*$Id: d_coment.cc,v 17.4 2000/03/19 04:20:02 al Exp $ -*- C++ -*-
  * processing for COMMENT netlist item (pseudo-device)
  * Old C style hack:
  * uses the space in a "card" struct, overwrites the leftover
@@ -12,13 +12,13 @@
 /*--------------------------------------------------------------------------*/
 void DEV_COMMENT::parse(CS& cmd)
 {
-  s = cmd.fullstring();
+  _s = cmd.fullstring();
 }
 /*--------------------------------------------------------------------------*/
-void DEV_COMMENT::print(OMSTREAM where, int)const
+void DEV_COMMENT::print(OMSTREAM& where, int)const
 {
-  if (s[0] != '*'  ||  s[1] != '+'){
-    where << s << '\n';
+  if (_s[0] != '*'  ||  _s[1] != '+'){
+    where << _s << '\n';
   }
   // Suppress printing of comment lines starting with "*+".
   // These are generated as a way to display calculated values.
