@@ -1,4 +1,4 @@
-/*$Id: d_coment.h,v 15.6 1999/09/20 06:10:44 al Exp $ -*- C++ -*-
+/*$Id: d_coment.h,v 17.4 2000/03/19 04:20:02 al Exp $ -*- C++ -*-
  $ comment statement
  */
 #ifndef D_COMENT_H
@@ -7,16 +7,17 @@
 /*--------------------------------------------------------------------------*/
 class DEV_COMMENT : public CARD {
 private:
-  std::string	s;
+  std::string	_s;
   explicit	DEV_COMMENT(const DEV_COMMENT& p)
 				:CARD(p) {untested(); set_constant(true);}
 public:
   explicit	DEV_COMMENT()	:CARD()  {set_constant(true);}
 private: // override virtual
+  char		id_letter()const{return '\0';}
   const char*	dev_type()const	{untested(); return "comment";}
   CARD*		clone()const	{untested(); return new DEV_COMMENT(*this);}
   void		parse(CS& cmd);
-  void		print(OMSTREAM where, int detail)const;
+  void		print(OMSTREAM& where, int detail)const;
 };
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
