@@ -1,4 +1,4 @@
-/*$Id: d_cs.h,v 15.11 1999/10/14 07:46:49 al Exp $ -*- C++ -*-
+/*$Id: d_cs.h,v 17.21 2000/05/02 09:32:08 al Exp $ -*- C++ -*-
  $ fixed current source
  */
 #ifndef D_CS_H
@@ -11,8 +11,11 @@ private:
 public:
   explicit DEV_CS()		:ELEMENT() {}
 private: // override virtual
+  char	   id_letter()const	{return 'I';}
   const char* dev_type()const	{return "isource";}
+  int	   numnodes()const	{return 2;}
   bool	   is_source()const	{return true;}
+  bool	   f_is_value()const	{return true;}
   CARD*	   clone()const		{return new DEV_CS(*this);}
   //void   parse(CS&);		//ELEMENT
   //void   print(OMSTREAM,int)const; //ELEMENT
@@ -31,11 +34,16 @@ private: // override virtual
   //double tr_review();		//CARD/nothing
   //void   tr_accept();		//CARD/nothing
   void	   tr_unload()		{tr_unload_source();}
+  //double tr_amps()const	//ELEMENT
   double   tr_involts()const	{return 0;}
+  double   tr_involts_limited()const {unreachable(); return 0;}
+  //double tr_probe_num(CS&)const;//ELEMENT
 
-  void	   ac_begin()		{acg = ev = 0.;}
+  void	   ac_begin()		{_acg = _ev = 0.;}
   void	   do_ac();
+  void	   ac_load()		{ac_load_source();}
   COMPLEX  ac_involts()const	{return 1.;}
+  //XPROBE ac_probe_ext(CS&)const;//ELEMENT
 };
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
