@@ -1,4 +1,4 @@
-/*$Id: d_dot.cc,v 14.19 1999/08/18 17:17:11 al Exp $ -*- C++ -*-
+/*$Id: d_dot.cc,v 17.23 2000/05/26 08:01:37 al Exp $ -*- C++ -*-
  * processing for DOTCARD netlist item (pseudo-device)
  * "parsing" a "dotcard" will execute it.
  * What actually happens (usually either to preset or really do)
@@ -13,7 +13,8 @@
 /*--------------------------------------------------------------------------*/
 void DEV_DOT::parse(CS& cmd)
 {
-  s = cmd.fullstring();
+  cmd.reset();
+  _s = cmd.fullstring();
   switch (ENV::run_mode){
   case rIGNORE:
     unreachable();
@@ -23,15 +24,15 @@ void DEV_DOT::parse(CS& cmd)
   case rINTERACTIVE:
   case rSCRIPT:
   case rBATCH:
-    cmd.skip();
+    cmd.skip1b(".");
     CMD::cmdproc(cmd.tail());
     break;
   }
 }
 /*--------------------------------------------------------------------------*/
-void DEV_DOT::print(OMSTREAM where, int)const
+void DEV_DOT::print(OMSTREAM& where, int)const
 {
-  where << s << '\n';
+  where << _s << '\n';
 }
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
