@@ -1,4 +1,4 @@
-/*$Id: d_dot.h,v 15.6 1999/09/20 06:10:44 al Exp $ -*- C++ -*-
+/*$Id: d_dot.h,v 17.4 2000/03/19 04:20:02 al Exp $ -*- C++ -*-
  $ a "dot card"
  */
 #ifndef D_DOT_H
@@ -7,16 +7,16 @@
 /*--------------------------------------------------------------------------*/
 class DEV_DOT : public CARD {
 private:
-  std::string	s;
-  explicit DEV_DOT(const DEV_DOT& p)
-				:CARD(p) {untested(); set_constant(true);}
+  std::string	_s;
+  explicit DEV_DOT(const DEV_DOT& p) :CARD(p) {untested(); set_constant(true);}
 public:
-  explicit	DEV_DOT()	:CARD()	 {set_constant(true);}
+  explicit	DEV_DOT()		:CARD() {set_constant(true);}
 private: // override virtual
-  const char*	dev_type()const	{untested(); return "dotcard";}
-  CARD*		clone()const	{untested(); return new DEV_DOT(*this);}
+  char		id_letter()const	{return '\0';}
+  const char*	dev_type()const		{untested(); return "dotcard";}
+  CARD*		clone()const		{untested();return new DEV_DOT(*this);}
   void		parse(CS&);
-  void		print(OMSTREAM,int)const;
+  void		print(OMSTREAM&,int)const;
 };
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
