@@ -1,17 +1,21 @@
-/*$Id: d_vcvs.h,v 15.11 1999/10/14 07:46:49 al Exp $ -*- C++ -*-
+/*$Id: d_vcvs.h,v 17.21 2000/05/02 09:32:08 al Exp $ -*- C++ -*-
  * voltage controled voltage source
  */
 #ifndef D_VCVS_H
 #define D_VCVS_H
-#include "e_vcsrc.h"
+#include "l_denoise.h"
+#include "e_elemnt.h"
 /*--------------------------------------------------------------------------*/
-class DEV_VCVS : public VCSRC_BASE {
+class DEV_VCVS : public ELEMENT {
 private:
-  explicit DEV_VCVS(const DEV_VCVS& p) :VCSRC_BASE(p) {}
+  explicit DEV_VCVS(const DEV_VCVS& p) :ELEMENT(p) {}
 public:
-  explicit DEV_VCVS()		:VCSRC_BASE() {}
+  explicit DEV_VCVS()		:ELEMENT() {}
 private: // override virtual
+  char	   id_letter()const	{return 'E';}
   const char* dev_type()const	{return "vcvs";}
+  int	   numnodes()const	{return 4;}
+  bool	   is_2port()const	{return true;}
   CARD*	   clone()const		{return new DEV_VCVS(*this);}
   //void   parse(CS&);		//ELEMENT
   //void   print(OMSTREAM,int)const; //ELEMENT
@@ -29,12 +33,17 @@ private: // override virtual
   void	   tr_load()		{tr_load_loss(); tr_load_active();}
   //double tr_review();		//CARD/nothing
   //void   tr_accept();		//CARD/nothing
-  //void   tr_unload();		//VCSRC_BASE
-  //double tr_involts()const;	//VCSRC_BASE
+  void	   tr_unload()		{tr_unload_active();}
+  //double tr_amps()const	//ELEMENT
+  double   tr_involts()const	{return dn_diff(_n[IN1].v0(), _n[IN2].v0());}
+  double   tr_involts_limited()const {return volts_limited(_n[IN1],_n[IN2]);}
+  //double tr_probe_num(CS&)const;//ELEMENT
 
-  void	    ac_begin()	{loss=1./OPT::shortckt; ev=y0.f1; acg=-loss*ev;}
-  void	    do_ac();
-  //COMPLEX ac_involts()const;	//VCSRC_BASE
+  void	   ac_begin();
+  void	   do_ac();
+  void	   ac_load()		{ac_load_loss(); ac_load_active();}
+  COMPLEX  ac_involts()const	{return _n[IN1].vac() - _n[IN2].vac();}
+  //XPROBE ac_probe_ext(CS&)const;//ELEMENT
 };
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
