@@ -1,4 +1,4 @@
-/*$Id: d_vs.h,v 15.11 1999/10/14 07:46:49 al Exp $ -*- C++ -*-
+/*$Id: d_vs.h,v 17.21 2000/05/02 09:32:08 al Exp $ -*- C++ -*-
  * fixed voltage source
  */
 #ifndef D_VS_H
@@ -11,8 +11,11 @@ private:
 public:
   explicit DEV_VS()		:ELEMENT() {}
 private: // override virtual
+  char	   id_letter()const	{return 'V';}
   const char* dev_type()const	{return "vsource";}
+  int	   numnodes()const	{return 2;}
   bool	   is_source()const	{return true;}
+  bool	   f_is_value()const	{return true;}
   CARD*	   clone()const		{return new DEV_VS(*this);}
   //void   parse(CS&);		//ELEMENT
   //void   print(OMSTREAM,int)const; //ELEMENT
@@ -31,11 +34,16 @@ private: // override virtual
   //double tr_review();		//CARD/nothing
   //void   tr_accept();		//CARD/nothing
   void	   tr_unload()		{tr_unload_source();}
+  //double tr_amps()const	//ELEMENT
   double   tr_involts()const	{return 0;}
+  double   tr_involts_limited()const {unreachable(); return 0;}
+  //double tr_probe_num(CS&)const;//ELEMENT
 
-  void	   ac_begin()		{loss = 1./OPT::shortckt; acg = ev = 0.;}
+  void	   ac_begin()	{_loss1 = _loss0 = 1./OPT::shortckt; _acg = _ev = 0.;}
   void	   do_ac();
+  void	   ac_load()		{ac_load_loss(); ac_load_source();}
   COMPLEX  ac_involts()const	{return 1.;}
+  //XPROBE ac_probe_ext(CS&)const;//ELEMENT
 };
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
