@@ -1,4 +1,4 @@
-/*$Id: e_aux.h,v 15.2 1999/09/17 14:24:57 al Exp $ -*- C++ -*-
+/*$Id: e_aux.h,v 17.1 2000/02/19 08:51:13 al Exp $ -*- C++ -*-
  * helper functions, etc., that sort of belong to circuit elements
  */
 #ifndef E_AUX_H
@@ -8,7 +8,8 @@
 /*--------------------------------------------------------------------------*/
 class node_t;
 /*--------------------------------------------------------------------------*/
-#if defined(COMPILE_TEMPLATES) || defined(ComTemP)
+#if defined(COMPILE_TEMPLATES)
+  #error untested
   #include "e_aux.cc"
 #else
   template <class T>
