@@ -1,4 +1,4 @@
-/*$Id: io_getln.cc,v 14.12 1999/07/23 10:13:54 al Exp $ -*- C++ -*-
+/*$Id: io_getln.cc,v 18.4 2000/07/09 05:58:31 al Exp $ -*- C++ -*-
  * get a bunch of lines, from a file
  * interface is just line fgets.
  * hooks together extension lines
@@ -6,7 +6,7 @@
  * is always a line ahead
  * start with + is extension line, spice compatibility.
  */
-#include "io_.h"
+#include "l_lib.h"
 // testing=nonstrict
 /*--------------------------------------------------------------------------*/
 std::string getlines(FILE *fileptr)
