@@ -1,4 +1,4 @@
-/*$Id: l_timer.h,v 13.44 1999/06/22 05:45:53 al Exp $ -*- C++ -*-
+/*$Id: l_timer.h,v 16.5 1999/11/25 07:56:24 al Exp $ -*- C++ -*-
  * CPU time accounting
  */
 #ifndef U_TIMER_H
@@ -25,8 +25,8 @@ public:
   TIMER&	start();
   TIMER&	stop();
   TIMER&	check();
-  double	elapsed()const	{untested(); return _last_user+_last_system;}
-  TIMER&	print(OMSTREAM s = IO::mstdout);
+  double	elapsed()const	{return _last_user+_last_system;}
+  TIMER&	print(OMSTREAM& s = IO::mstdout);
   TIMER& 	operator=(const TIMER&);
   friend TIMER	operator-(const TIMER&,const TIMER&);
 };
