@@ -1,8 +1,7 @@
-/*$Id: l_trim.cc,v 13.44 1999/06/22 05:45:53 al Exp $ -*- C++ -*-
+/*$Id: l_trim.cc,v 18.5 2000/07/10 08:36:32 al Exp $ -*- C++ -*-
  * remove whitespace from the end of strings
  */
-#include "md.h"
-#include "l_lib.h"
+#include <ctype.h>
 /*--------------------------------------------------------------------------*/
 	char*	trim(char*);
 /*--------------------------------------------------------------------------*/
