@@ -1,4 +1,4 @@
-/*$Id: s__.cc,v 15.19 1999/11/03 18:07:50 al Exp $ -*- C++ -*-
+/*$Id: s__.cc,v 18.5 2000/07/10 08:36:32 al Exp $ -*- C++ -*-
  * base class for simulation methods
  */
 #include "s__.h"
@@ -16,7 +16,7 @@ bool	SIM::bypass_ok = false;
 TRI_STATE SIM::inc_mode = tsNO;
 bool	SIM::fulldamp = false;
 int	SIM::mode = sNONE;
-sim_phase_t SIM::phase = pNONE;
+SIM::PHASE SIM::phase = pNONE;
 bool	SIM::freezetime = false;
 int*	SIM::nm = NULL;
 double*	SIM::i = NULL;
